/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "vacuum.h"

// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

static void vacuum_read ( vacuum_t *ctx, uint8_t *data_buf, uint8_t len );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void vacuum_cfg_setup ( vacuum_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->scl = HAL_PIN_NC;
    cfg->sda = HAL_PIN_NC;
    
    // Additional gpio pins

    cfg->an   = HAL_PIN_NC;
  
    cfg->i2c_speed = I2C_STANDARD_SPEED; 
    cfg->i2c_address = 0x4D;
}

VACCUM_RETVAL vacuum_init ( vacuum_t *ctx, vacuum_cfg_t *cfg )
{
    i2c_master_config_t i2c_cfg;

    i2c_master_configure_default( &i2c_cfg );
    i2c_cfg.speed    = cfg->i2c_speed;
    i2c_cfg.pins.scl = cfg->scl;
    i2c_cfg.pins.sda = cfg->sda;

    ctx->slave_address = cfg->i2c_address;

    if ( i2c_master_open( &ctx->i2c, &i2c_cfg ) != I2C_SUCCESS )
    {
        return VACCUM_INIT_ERROR;
    }

    // Output pins 

    digital_out_init( &ctx->an, cfg->an );

    return VACCUM_OK;
}

void vacuum_generic_write ( vacuum_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    i2c_master_transfer_t i2c_transfer;
    uint8_t tx_buf[ 256 ];
    uint8_t cnt;
    
    tx_buf[ 0 ] = reg;
    
    for ( cnt = 1; cnt <= len; cnt++ )
    {
        tx_buf[ cnt ] = *data_buf;
        data_buf++; 
    }
    
    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = tx_buf;
    i2c_transfer.count    = len + 1;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_start( &ctx->i2c );
    i2c_master_write( &ctx->i2c, &i2c_transfer );    
}

void vacuum_generic_read ( vacuum_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    i2c_master_transfer_t i2c_transfer;
    uint8_t tx_data;

    tx_data = reg;
    
    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = &tx_data;
    i2c_transfer.count    = 1;
    i2c_transfer.end_mode = I2C_RESTART_MODE;

    i2c_master_start( &ctx->i2c );
    i2c_master_write( &ctx->i2c, &i2c_transfer );
    
    i2c_transfer.buffer   = data_buf;
    i2c_transfer.count    = len;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_read( &ctx->i2c, &i2c_transfer );
}

uint16_t vacuum_read_data ( vacuum_t *ctx )
{
    uint8_t read_reg[ 2 ];
    uint16_t read_data;

    vacuum_read( ctx, read_data, 2 );

    read_data = read_reg[ 0 ];
    read_data = read_data << 8; 
    read_data = read_data | read_reg[ 1 ];  
    
    return read_data;
}

float vacuum_get_voltage ( vacuum_t *ctx )
{
    uint16_t read_data;
    float voltage;

    vacuum_read( ctx, read_data, 1 );

    read_data = vacuum_read_data( ctx );
    voltage = (read_data * 5.0) / 4096.0;
    
    return voltage;
}

float vacuum_get_pressure ( vacuum_t *ctx )
{
    
    float pressure_m_bar;

    ctx->pressure = vacuum_read_data( ctx );
    pressure_m_bar = ( ( ctx->pressure - 2400.0 ) / ( ( 13107.0 ) / 100.0 ) );
    pressure_m_bar *= 100.0;
    
    return pressure_m_bar;
}

float vacuum_get_percentage_of_vacuum ( vacuum_t *ctx )
{
    float pressure_m_bar;
    float vacuum;
    
    pressure_m_bar = vacuum_get_pressure( ctx );
    vacuum = ( ctx->pressure - pressure_m_bar ) / 18.0;
    
    if ( vacuum < 1 )
    {
        return 0;
    }
    else if ( vacuum > 100 )
    {
        return 100;
    }
    
    return vacuum;
}

void vacuum_calibration ( vacuum_t *ctx )
{
    float pressure_m_bar;
    ctx->pressure = vacuum_get_pressure( ctx );
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void vacuum_read ( vacuum_t *ctx, uint8_t *data_buf, uint8_t len )
{
    i2c_master_transfer_t i2c_transfer;

    i2c_transfer.slave_address = ctx->slave_address;
    
    i2c_master_start( &ctx->i2c );
    
    i2c_transfer.buffer   = data_buf;
    i2c_transfer.count    = len;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_read( &ctx->i2c, &i2c_transfer );
}
// ------------------------------------------------------------------------- END

